import json

import django.test
from django.conf import settings


class TestCase(django.test.TestCase):
    test_data_base_dir = settings.BASE_DIR

    def json_data(self, *path):
        path = self.test_data_base_dir.joinpath(*path)
        with path.open() as fp:
            return json.load(fp)

    def raw_data(self, *path):
        path = self.test_data_base_dir.joinpath(*path)
        with path.open() as fp:
            return fp.read()



import json
from urllib.parse import urlencode

from django import template
from django.shortcuts import reverse

# The JSONEncoder from DRF handles quite a few types, so we default to that
# if available and if not fallback to the Django one which still handles some
# extra types
try:
    from rest_framework.utils.encoders import JSONEncoder
except ImportError:
    from django.core.serializers.json import DjangoJSONEncoder as JSONEncoder


register = template.Library()


@register.simple_tag(takes_context=True)
def qs(context, *args, **kwargs):
    """
    Query string handler for paginators and filters

    Assuming we have a query string like ?page=1&search=foo, there are
    several cases in which we want to replace only the page key, while leaving
    the rest alone. This tag allows us to replace individual values (like the
    current page) while carrying over other values (like a search string)

    Example:
    <a href="?{% qs 'page' page_obj.next_page_number %}">
    <a href="?{% qs foo=bar %}">
    """
    qs = context["request"].GET.copy()
    # We special case when args are passed, because the first arg may
    # be the name of a key we want to replace. If an item is pased as
    # args, we'll add it to our kwargs so the rest of our code is simple
    if args:
        kwargs[args[0]] = args[1]
    for key in kwargs:
        if kwargs[key] is None:
            qs.pop(key, None)
        else:
            qs[key] = kwargs[key]
    return urlencode(qs)


@register.filter(name="prettyjson")
def prettyjson(value):
    if isinstance(value, str):
        value = json.loads(value)
    return json.dumps(
        value,
        indent=2,
        sort_keys=True,
        cls=JSONEncoder,
    )


@register.simple_tag(takes_context=True)
def fullurl(context, viewname, *args, **kwargs):
    return context["request"].build_absolute_uri(
        reverse(viewname, args=args, kwargs=kwargs)
    )



import requests

from django.conf import settings
from django.contrib.sites.shortcuts import get_current_site
from functools import wraps

try:
    from importlib.metadata import version, PackageNotFoundError
except ImportError:
    from importlib_metadata import version, PackageNotFoundError

DEFAULT_DISTRIBUTION = getattr(settings, "USER_AGENT_DISTRIBUTION", "django-zakka")


def user_agent(name):
    try:
        v = version(distribution_name=name)
    except PackageNotFoundError:
        v = "unknown"

    try:
        domain = get_current_site(None).domain
    except Exception:
        return f"{name}/{v}"
    else:
        return f"{name}/{v} (+{domain})"


class DjangoSession(requests.Session):
    def __init__(self, distribution_name=DEFAULT_DISTRIBUTION):
        super().__init__()
        self.headers["user-agent"] = user_agent(distribution_name)


USER_AGENT = getattr(settings, "USER_AGENT", user_agent(DEFAULT_DISTRIBUTION))


@wraps(requests.request, assigned=["__doc__"])
def request(method, url, **kwargs):
    with DjangoSession() as session:
        return session.request(method=method, url=url, **kwargs)


@wraps(requests.get, assigned=["__doc__"])
def get(url, **kwargs) -> requests.Response:
    return request("get", url, **kwargs)


@wraps(requests.head, assigned=["__doc__"])
def head(url, **kwargs) -> requests.Response:
    return request("head", url, **kwargs)


@wraps(requests.options, assigned=["__doc__"])
def options(url, **kwargs) -> requests.Response:
    return request("options", url, **kwargs)


@wraps(requests.post, assigned=["__doc__"])
def post(url, data=None, json=None, **kwargs) -> requests.Response:
    return request("post", url, data=data, json=json, **kwargs)


@wraps(requests.put, assigned=["__doc__"])
def put(url, data=None, **kwargs) -> requests.Response:
    return request("put", url, data=data, **kwargs)


@wraps(requests.patch, assigned=["__doc__"])
def patch(url, data=None, **kwargs) -> requests.Response:
    return request("patch", url, data=data, **kwargs)


@wraps(requests.delete, assigned=["__doc__"])
def delete(url, **kwargs) -> requests.Response:
    return request("delete", url, **kwargs)



from django.contrib.auth import mixins


class SuperuserRequiredMixin(mixins.AccessMixin):
    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_superuser:
            return self.handle_no_permission()
        return super().dispatch(request, *args, **kwargs)



from pprint import pprint

from celery import current_app

from django.core.management.base import BaseCommand
from django.test import override_settings

from zakka.mixins.command import LoggingMixin


class Command(LoggingMixin, BaseCommand):
    def add_arguments(self, parser):
        # Positional arguments
        parser.add_argument("--eager", action="store_true")
        parser.add_argument("task", nargs="?")
        parser.add_argument("rest", nargs="*")

    def handle(self, task, eager, rest, **options):
        current_app.loader.import_default_modules()
        if task in current_app.tasks:
            with override_settings(CELERY_TASK_ALWAYS_EAGER=eager):
                result = current_app.tasks[task](*rest)
                if hasattr(result, "__iter__"):
                    result = list(result)
                pprint(result)
        else:
            for task in sorted(current_app.tasks):
                print(task)



from celery import current_app
from celery.execute import send_task

from django import forms
from django.contrib import messages
from django.urls import path
from django.views.generic import FormView

from zakka import permissions


class TaskForm(forms.Form):
    task = forms.CharField(label="Your name", max_length=100)


class CeleryJobs(permissions.SuperuserRequiredMixin, FormView):
    template_name = "zakka/celery/tasks.html"
    form_class = TaskForm

    def get_context_data(self, **kwargs):
        data = super().get_context_data(**kwargs)
        current_app.loader.import_default_modules()
        data["tasks"] = current_app.tasks
        return data

    def form_valid(self, form):
        messages.add_message(
            self.request, messages.INFO, "Running %s" % form.data["task"]
        )
        send_task(form.data["task"])
        return self.get(self.request)

    def form_invalid(self, form):
        messages.add_message(self.request, messages.ERROR, "Invalid %s" % form.errors)
        return self.get(self.request)


urlpatterns = [path("", CeleryJobs.as_view())]



from .client import (  # NOQA
    request,
    get,
    head,
    post,
    patch,
    put,
    delete,
    options,
)
from requests.status_codes import codes  # NOQA
from requests.exceptions import (  # NOQA
    RequestException,
    Timeout,
    URLRequired,
    TooManyRedirects,
    HTTPError,
    ConnectionError,
    FileModeWarning,
    ConnectTimeout,
    ReadTimeout,
    JSONDecodeError,
)



from celery import current_app
from celery.app.task import Task
from celery.schedules import maybe_schedule


class PeriodicTask(Task):
    relative = False
    options = None

    def __init__(self):
        if not hasattr(self, "run_every"):
            raise NotImplementedError("Periodic tasks must have a run_every attribute")
        self.run_every = maybe_schedule(self.run_every, self.relative)
        super().__init__()

    @classmethod
    def on_bound(cls, app):
        app.conf.beat_schedule[cls.name] = {
            "task": cls.name,
            "schedule": cls.run_every,
            "args": (),
            "kwargs": {},
            "options": cls.options or {},
            "relative": cls.relative,
        }


def periodic_task(*args, **kwargs):
    return current_app.task(*args, **dict({"base": PeriodicTask}, **kwargs))
